@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2017, University of Amsterdam
+    Copyright (c)  1985-2020, University of Amsterdam
                               VU University Amsterdam
     All rights reserved.
 
@@ -499,7 +499,7 @@ initialise_prolog :-
     '$set_file_search_paths',
     init_debug_flags,
     start_pldoc,
-    attach_packs,
+    opt_attach_packs,
     '$cmd_option_val'(init_file, OsFile),
     prolog_to_os_filename(File, OsFile),
     '$load_init_file'(File),
@@ -519,6 +519,12 @@ initialise_prolog :-
         )
     ).
 
+opt_attach_packs :-
+    current_prolog_flag(packs, true),
+    !,
+    attach_packs.
+opt_attach_packs.
+
 set_toplevel :-
     '$cmd_option_val'(toplevel, TopLevelAtom),
     catch(term_to_atom(TopLevel, TopLevelAtom), E,
@@ -590,15 +596,16 @@ run_init_goal(Goal, Ctx) :-
 
 init_debug_flags :-
     once(print_predicate(_, [print], PrintOptions)),
-    create_prolog_flag(answer_write_options, PrintOptions, []),
-    create_prolog_flag(prompt_alternatives_on, determinism, []),
-    create_prolog_flag(toplevel_extra_white_line, true, []),
-    create_prolog_flag(toplevel_print_factorized, false, []),
+    Keep = [keep(true)],
+    create_prolog_flag(answer_write_options, PrintOptions, Keep),
+    create_prolog_flag(prompt_alternatives_on, determinism, Keep),
+    create_prolog_flag(toplevel_extra_white_line, true, Keep),
+    create_prolog_flag(toplevel_print_factorized, false, Keep),
     create_prolog_flag(print_write_options,
                        [ portray(true), quoted(true), numbervars(true) ],
-                       []),
-    create_prolog_flag(toplevel_residue_vars, false, []),
-    create_prolog_flag(toplevel_list_wfs_residual_program, true, []),
+                       Keep),
+    create_prolog_flag(toplevel_residue_vars, false, Keep),
+    create_prolog_flag(toplevel_list_wfs_residual_program, true, Keep),
     '$set_debugger_write_options'(print).
 
 %!  setup_backtrace
@@ -743,7 +750,7 @@ setup_interactive :-
     '$set_file_search_paths',
     init_debug_flags,
     '$run_initialization',
-    attach_packs,
+    opt_attach_packs,
     use_module(library(qsave)),
     qsave:qsave_toplevel.
 
@@ -850,9 +857,14 @@ read_query(Prompt, Goal, Bindings) :-
     current_prolog_flag(history, N),
     integer(N), N > 0,
     !,
-    read_history(h, '!h',
-                 [trace, end_of_file],
-                 Prompt, Goal, Bindings).
+    read_term_with_history(
+        Goal,
+        [ show(h),
+          help('!h'),
+          no_save([trace, end_of_file]),
+          prompt(Prompt),
+          variable_names(Bindings)
+        ]).
 read_query(Prompt, Goal, Bindings) :-
     remove_history_prompt(Prompt, Prompt1),
     repeat,                                 % over syntax errors
