@@ -52,7 +52,7 @@
 :- autoload(library(option),[option/2,option/3,meta_options/3]).
 :- autoload(library(prolog_clause),[clause_info/5]).
 
-:- set_prolog_flag(generate_debug_info, false).
+%:- set_prolog_flag(generate_debug_info, false).
 
 :- module_transparent
     listing/0.
@@ -376,8 +376,18 @@ write_declarations([H|T], Module) :-
 
 list_clauses(Pred, Source, Options) :-
     strip_module(Pred, Module, Head),
-    forall(clause(Pred, Body, Ref),
-           list_clause(Module:Head, Body, Ref, Source, Options)).
+    generalise_term(Head, GenHead),
+    forall(( clause(Module:GenHead, Body, Ref),
+             \+ GenHead \= Head
+           ),
+           list_clause(Module:GenHead, Body, Ref, Source, Options)).
+
+generalise_term(Head, Gen) :-
+    compound(Head),
+    !,
+    compound_name_arity(Head, Name, Arity),
+    compound_name_arity(Gen,  Name, Arity).
+generalise_term(Head, Head).
 
 list_clause(_Head, _Body, Ref, _Source, Options) :-
     option(source(true), Options),
@@ -587,15 +597,29 @@ close_sources :-
 %!  portray_clause(+Out:stream, +Clause) is det.
 %!  portray_clause(+Out:stream, +Clause, +Options) is det.
 %
-%   Portray `Clause' on the current  output   stream.  Layout of the
-%   clause is to our best standards.   As  the actual variable names
-%   are not available we use A, B, ... Deals with ';', '|', '->' and
-%   calls via meta-call predicates as determined using the predicate
-%   property   meta_predicate.   If   Clause   contains   attributed
-%   variables, these are treated as normal variables.
+%   Portray `Clause' on the current output  stream. Layout of the clause
+%   is to our best standards. Deals   with  control structures and calls
+%   via meta-call predicates as determined  using the predicate property
+%   meta_predicate. If Clause contains attributed   variables, these are
+%   treated as normal variables.
 %
-%   If  Options  is  provided,   the    option-list   is  passed  to
-%   write_term/3 that does the final writing of arguments.
+%   Variable names are by default generated using numbervars/4 using the
+%   option singletons(true). This names the variables  `A`, `B`, ... and
+%   the singletons `_`. Variables can  be   named  explicitly by binding
+%   them to a term `'$VAR'(Name)`, where `Name`   is  an atom denoting a
+%   valid  variable  name  (see   the    option   numbervars(true)  from
+%   write_term/2) as well  as  by   using  the  variable_names(Bindings)
+%   option from write_term/2.
+%
+%   Options processed in addition to write_term/2 options:
+%
+%     - variable_names(+Bindings)
+%       See above and write_term/2.
+%     - indent(+Columns)
+%       Left margin used for the clause.  Default `0`.
+%     - module(+Module)
+%       Module used to determine whether a goal resolves to a meta
+%       predicate.  Default `user`.
 
 %       The prolog_list_goal/1 hook is  a  dubious   as  it  may lead to
 %       confusion if the heads relates to other   bodies.  For now it is
@@ -616,12 +640,34 @@ portray_clause(Stream, Term) :-
 portray_clause(Stream, Term, M:Options) :-
     must_be(list, Options),
     meta_options(is_meta, M:Options, QOptions),
-    \+ \+ ( copy_term_nat(Term, Copy),
-            numbervars(Copy, 0, _,
-                       [ singletons(true)
-                       ]),
-            do_portray_clause(Stream, Copy, QOptions)
-          ).
+    \+ \+ name_vars_and_portray_clause(Stream, Term, QOptions).
+
+name_vars_and_portray_clause(Stream, Term, Options) :-
+    term_attvars(Term, []),
+    !,
+    clause_vars(Term, Options),
+    do_portray_clause(Stream, Term, Options).
+name_vars_and_portray_clause(Stream, Term, Options) :-
+    option(variable_names(Bindings), Options),
+    !,
+    copy_term_nat(Term+Bindings, Copy+BCopy),
+    bind_vars(BCopy),
+    name_other_vars(Copy, BCopy),
+    do_portray_clause(Stream, Copy, Options).
+name_vars_and_portray_clause(Stream, Term, Options) :-
+    copy_term_nat(Term, Copy),
+    clause_vars(Copy, Options),
+    do_portray_clause(Stream, Copy, Options).
+
+clause_vars(Clause, Options) :-
+    option(variable_names(Bindings), Options),
+    !,
+    bind_vars(Bindings),
+    name_other_vars(Clause, Bindings).
+clause_vars(Clause, _) :-
+    numbervars(Clause, 0, _,
+               [ singletons(true)
+               ]).
 
 is_meta(portray_goal).
 
@@ -663,23 +709,17 @@ do_portray_clause(Out, Term, Options) :-
         portray_body(Body, BodyIndent, indent, RightPri, Out, Options)
     ),
     full_stop(Out).
-do_portray_clause(Out, (:-use_module(File, Imports)), Options) :-
-    length(Imports, Len),
-    Len > 3,
+do_portray_clause(Out, (:-Directive), Options) :-
+    wrapped_list_directive(Directive),
     !,
+    Directive =.. [Name, Arg, List],
     option(indent(LeftMargin), Options, 0),
     indent(Out, LeftMargin),
-    ListIndent is LeftMargin+14,
-    format(Out, ':- use_module(~q,', [File]),
-    portray_list(Imports, ListIndent, Out, Options),
-    write(Out, ').\n').
-do_portray_clause(Out, (:-module(Module, Exports)), Options) :-
-    !,
-    option(indent(LeftMargin), Options, 0),
-    indent(Out, LeftMargin),
-    ModuleIndent is LeftMargin+10,
-    format(Out, ':- module(~q,', [Module]),
-    portray_list(Exports, ModuleIndent, Out, Options),
+    format(Out, ':- ~q(', [Name]),
+    line_position(Out, Indent),
+    format(Out, '~q,', [Arg]),
+    nlindent(Out, Indent),
+    portray_list(List, Indent, Out, Options),
     write(Out, ').\n').
 do_portray_clause(Out, (:-Directive), Options) :-
     !,
@@ -702,6 +742,9 @@ full_stop(Out) :-
     '$put_token'(Out, '.'),
     nl(Out).
 
+wrapped_list_directive(module(_,_)).
+%wrapped_list_directive(use_module(_,_)).
+%wrapped_list_directive(autoload(_,_)).
 
 %!  portray_body(+Term, +Indent, +DoIndent, +Priority, +Out, +Options)
 %
@@ -741,6 +784,14 @@ portray_body(Term, Indent, _, Pri, Out, Options) :-
     portray_body(Term, ArgIndent, noindent, 1200, Out, Options),
     nlindent(Out, Indent),
     write(Out, ')').
+portray_body(((AB),C), Indent, _, _Pri, Out, Options) :-
+    nonvar(AB),
+    AB = (A,B),
+    !,
+    infix_op(',', LeftPri, RightPri),
+    portray_body(A, Indent, noindent, LeftPri, Out, Options),
+    write(Out, ','),
+    portray_body((B,C), Indent, indent, RightPri, Out, Options).
 portray_body((A,B), Indent, _, _Pri, Out, Options) :-
     !,
     infix_op(',', LeftPri, RightPri),
@@ -959,7 +1010,6 @@ portray_list([], _, Out, _) :-
     !,
     write(Out, []).
 portray_list(List, Indent, Out, Options) :-
-    nlindent(Out, Indent),
     write(Out, '[ '),
     EIndent is Indent + 2,
     portray_list_elements(List, EIndent, Out, Options),
@@ -1021,7 +1071,18 @@ pprint(Out, Term, Pri, Options) :-
     pprint_wrapped(Out, Term, Pri, Options).
 pprint(Out, Term, Pri, Options) :-
     listing_write_options(Pri, WrtOptions, Options),
-    write_term(Out, Term, WrtOptions).
+    write_term(Out, Term,
+               [ blobs(portray),
+                 portray_goal(portray_blob)
+               | WrtOptions
+               ]).
+
+portray_blob(Blob, _Options) :-
+    blob(Blob, _),
+    \+ atom(Blob),
+    !,
+    format(string(S), '~q', [Blob]),
+    format('~q', ['$BLOB'(S)]).
 
 nowrap_term('$VAR'(_)) :- !.
 nowrap_term(_{}) :- !.                  % empty dict
