@@ -111,7 +111,7 @@ get_optval(optvalue valp, const opt_spec *spec, term_t val ARG_LD)
     case OPT_STRING:
     { char *str;
 
-      if ( !PL_get_chars(val, &str, CVT_ALL|REP_UTF8|BUF_RING|CVT_EXCEPTION) )
+      if ( !PL_get_chars(val, &str, CVT_ALL|REP_UTF8|BUF_STACK|CVT_EXCEPTION) )
 	return FALSE;
       *valp.s = str;
 
@@ -260,7 +260,7 @@ scan_options(term_t options, int flags, atom_t optype,
     { return PL_error(NULL, 0, NULL, ERR_INSTANTIATION);
     } else
     { itemerror:
-      return PL_error(NULL, 0, NULL, ERR_DOMAIN, optype, head);
+      return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_option, head);
     }
 
     for( n=0, s = specs; s->name; n++, s++ )
