@@ -3,7 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2011-2016, University of Amsterdam
+    Copyright (c)  2011-2020, University of Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -35,7 +36,7 @@
 /*#define O_DEBUG 1*/
 #include "pl-incl.h"
 #include "pl-copyterm.h"
-#define AC_TERM_WALK_LR 1
+#define AC_TERM_WALK_LRD 1
 #include "pl-termwalk.c"
 
 
@@ -393,20 +394,22 @@ exitCyclicCopy(int flags ARG_LD)
 
 static int
 copy_term(Word from, Word to, size_t abstract, int flags ARG_LD)
-{ term_agendaLR agenda;
+{ term_agendaLRD agenda;
   int rc = TRUE;
+  size_t aleft = (size_t)-1;
+
+  initTermAgendaLRD(&agenda, 1, from, to);
+  while( nextTermAgendaLRD(&agenda, &from, &to) )
+  { if ( agenda.work.depth == 1 )
+      aleft = abstract;
 
-  initTermAgendaLR(&agenda, 1, from, to);
-  while( nextTermAgendaLR(&agenda, &from, &to) )
-  {
   again:
-
     switch(tag(*from))
     { case TAG_REFERENCE:
       { Word p2 = unRef(*from);
 
 	if ( *p2 == VAR_MARK )		/* reference to a copied variable */
-	{ *to = makeRef(p2);
+	{ *to = makeRefG(p2);
 	} else
 	{ from = p2;			/* normal reference */
 	  goto again;
@@ -417,10 +420,10 @@ copy_term(Word from, Word to, size_t abstract, int flags ARG_LD)
       case TAG_VAR:
       { if ( shared(*from) )
 	{ *to = VAR_MARK;
-	  *from = makeRef(to);
+	  *from = makeRefG(to);
 	  TrailCyclic(from PASS_LD);
 	} else if ( (flags&COPY_ABSTRACT) )
-	{ *to = makeRef(from);
+	{ *to = makeRefG(from);
 	} else
 	{ setVar(*to);
 	}
@@ -454,7 +457,7 @@ copy_term(Word from, Word to, size_t abstract, int flags ARG_LD)
 	  { Word p = valPAttVar(*from & ~BOTH_MASK);
 
 	    if ( *p == VAR_MARK )
-	    { *to = makeRef(p);
+	    { *to = makeRefG(p);
 	    } else
 	    { *to = VAR_MARK;
 	      *from = consPtr(to, STG_GLOBAL|TAG_ATTVAR)|BOTH_MASK;
@@ -469,12 +472,12 @@ copy_term(Word from, Word to, size_t abstract, int flags ARG_LD)
       case TAG_COMPOUND:
       { Functor ff = valueTerm(*from);
 
-	if ( abstract == 0 )
+	if ( aleft == 0 )
 	{ setVar(*to);
 	  continue;
 	} else
-	{ if ( abstract != (size_t)-1 )
-	    abstract--;
+	{ if ( aleft != (size_t)-1 )
+	    aleft--;
 	}
 
 	if ( isRef(ff->definition) )
@@ -500,7 +503,7 @@ copy_term(Word from, Word to, size_t abstract, int flags ARG_LD)
 	  TrailCyclic(&ff->definition PASS_LD);
 	  *to = consPtr(ft, TAG_COMPOUND|STG_GLOBAL);
 
-	  if ( pushWorkAgendaLR(&agenda, arity, ff->arguments, ft->arguments) )
+	  if ( pushWorkAgendaLRD(&agenda, arity, ff->arguments, ft->arguments) )
 	    continue;
 	  rc = MEMORY_OVERFLOW;
 	  goto out;
@@ -515,12 +518,15 @@ copy_term(Word from, Word to, size_t abstract, int flags ARG_LD)
 	  ft->definition = ff->definition & ~BOTH_MASK;
 	  *to = consPtr(ft, TAG_COMPOUND|STG_GLOBAL);
 
-	  if ( pushWorkAgendaLR(&agenda, arity, ff->arguments, ft->arguments) )
+	  if ( pushWorkAgendaLRD(&agenda, arity, ff->arguments, ft->arguments) )
 	    continue;
 	  rc = MEMORY_OVERFLOW;
 	  goto out;
 	}
       }
+      case TAG_ATOM:
+	pushVolatileAtom(*from);
+        /*FALLTHROUGH*/
       default:
 	*to = *from;
         continue;
@@ -528,7 +534,7 @@ copy_term(Word from, Word to, size_t abstract, int flags ARG_LD)
   }
 
 out:
-  clearTermAgendaLR(&agenda);
+  clearTermAgendaLRD(&agenda);
   return rc;
 }
 
@@ -552,6 +558,9 @@ again:
     case TAG_ATTVAR:
     case TAG_COMPOUND:
       break;
+    case TAG_ATOM:
+      pushVolatileAtom(*from);
+      /*FALLTHROUGH*/
     default:
       *to = *from;
       return TRUE;
@@ -595,7 +604,7 @@ copy_term_refs(term_t from, term_t to, size_t abstract, int flags ARG_LD)
       return FALSE;			/* stack */
     }
     setVar(*dest);
-    *valTermRef(to) = makeRef(dest);
+    *valTermRef(to) = makeRefG(dest);
     src = valTermRef(from);
 
     rc = do_copy_term(src, dest, abstract, flags PASS_LD);
